#include "datalogview.h"
#include <QHeaderView>
#include <QTableWidgetItem>
#include <QDebug>

dataLogView::dataLogView(QWidget *parent):QTableWidget(parent)
{
    verticalHeader()->hide();
    verticalHeader()->setDefaultSectionSize(60);

    QStringList headerLabels;
    headerLabels << "Parameter" << "Data" << "Cursor" << "Min" << "Max" << "Mean" << "Std Dev";
    setColumnCount(headerLabels.count());
    setHorizontalHeaderLabels(headerLabels);

    horizontalHeader()->setSectionResizeMode(0, QHeaderView::ResizeToContents);
    horizontalHeader()->setSectionResizeMode(1, QHeaderView::Stretch);
    horizontalHeader()->setSectionResizeMode(2, QHeaderView::ResizeToContents);
    horizontalHeader()->setSectionResizeMode(3, QHeaderView::ResizeToContents);
    horizontalHeader()->setSectionResizeMode(4, QHeaderView::ResizeToContents);
    horizontalHeader()->setSectionResizeMode(5, QHeaderView::ResizeToContents);
    horizontalHeader()->setSectionResizeMode(6, QHeaderView::ResizeToContents);
}

void dataLogView::slotSetSamples(const QVector<QPair<QString, QVector<QPointF> > > &samples)
{
    //headerLabels << "Parameter" << "Data" << "Cursor" << "Min" << "Max" << "Mean" << "Std Dev";
    setRowCount(samples.count());
    for(int i=0; i < samples.count(); ++i)
    {
        const QPair<QString, QVector<QPointF> >& sample = samples[i];

        // fill out parameter column
        QTableWidgetItem* item = new QTableWidgetItem(sample.first);
        item->setTextAlignment(Qt::AlignCenter);
        setItem(i, 0, item);

        // add plot to data column
        dataPlot* plot = new dataPlot(this);
        const QVector<QPointF>& plotData = sample.second;

        plot->slotSetData(plotData);
        setCellWidget(i, 1, plot);

        // add empty item to cursor column
        item = new QTableWidgetItem("");
        item->setTextAlignment(Qt::AlignCenter);
        setItem(i, 2, item);

        QVector<double> values;
        for(int i=0; i < plotData.count(); ++i)
        {
            values.append(plotData[i].y());
        }

        if(values.count() == 0)
        {
            item = new QTableWidgetItem("-");
            item->setTextAlignment(Qt::AlignCenter);
            setItem(i, 3, item);

            item = new QTableWidgetItem("-");
            item->setTextAlignment(Qt::AlignCenter);
            setItem(i, 4, item);

            item = new QTableWidgetItem("-");
            item->setTextAlignment(Qt::AlignCenter);
            setItem(i, 5, item);

            item = new QTableWidgetItem("-");
            item->setTextAlignment(Qt::AlignCenter);
            setItem(i, 6, item);
        } else {
            double min = calcMin(values);
            double max = calcMax(values);
            double mean = calcMean(values);
            double stddev = calcStdDev(values, mean);

            // calculate and insert Min
            item = new QTableWidgetItem(QString::number(min, 'f', 1));
            item->setTextAlignment(Qt::AlignCenter);
            setItem(i, 3, item);

            // calculate and insert Max
            item = new QTableWidgetItem(QString::number(max, 'f', 1));
            item->setTextAlignment(Qt::AlignCenter);
            setItem(i, 4, item);

            // calculate and insert Mean
            item = new QTableWidgetItem(QString::number(mean, 'f', 2));
            item->setTextAlignment(Qt::AlignCenter);
            setItem(i, 5, item);

            // calculate and insert Std Dev
            item = new QTableWidgetItem(QString::number(stddev, 'f', 2));
            item->setTextAlignment(Qt::AlignCenter);
            setItem(i, 6, item);
        }
    }
}

double dataLogView::calcMin(const QVector<double> &values) const
{
    double min = std::numeric_limits<double>::max();
    for(int i=0; i < values.count(); ++i)
    {
        const double& val = values[i];
        if(val < min)
        {
            min = val;
        }
    }
    return min;
}

double dataLogView::calcMax(const QVector<double> &values) const
{
    double max = std::numeric_limits<double>::min();
    for(int i=0; i < values.count(); ++i)
    {
        const double& val = values[i];
        if(val > max)
        {
            max = val;
        }
    }
    return max;
}

double dataLogView::calcMean(const QVector<double> &values) const
{
    double sum = 0.0;
    QVectorIterator<double> iter(values);
    while(iter.hasNext())
    {
        sum += iter.next();
    }
    double mean = 0.0;
    if(values.count() != 0)
    {
        mean = sum / ((double) values.count());
    }
    return mean;
}

double dataLogView::calcStdDev(const QVector<double> &values, const double &mean) const
{
    if (values.size() == 0 || mean == 0.0)
    {
        return 0.0;
    }
    double sumOfSquares = 0.0;
    double stdDev = 0.0;
    QVectorIterator<double> iter(values);
    while(iter.hasNext())
    {
        sumOfSquares += pow(mean - iter.next(), 2);
    }
    double meanOfSS = sumOfSquares / ((double) values.count());
    stdDev = sqrt(meanOfSS);
    return stdDev;
}
